import sys
import functools
import datetime
from pprint import pformat
from webob import Request, Response, exc
from user import User
from session import Session

import bivouac

def action(func):

    '''MVC @action decorator as a WSGI application.
    '''

    def replacement(self, environ, start_response):

        req = Request(environ)

        if self.require_auth:
            self.user = self.get_user(req)
            if self.user is None:
                resp = self.redirect('/login')
                return resp(environ, start_response)
            
        try:
            resp = func(self, req, **req.urlvars)   # original function called here
        except exc.HTTPException, e:
            resp = e

        if isinstance(resp, basestring):
            resp = Response(body=resp)
            
        return resp(environ, start_response)

    return replacement


def noauth(func):

    '''MVC @noauth decorator as a WSGI middleware application.
    @noauth prevents checking for user.  Used for "public" pages.
    '''
    
    def replacement(self, environ, start_response):
        self.require_auth = False
        return func(self, environ, start_response)

    return replacement
        
    

class Controller(object):

    '''MVC Controller for WSGI applications.
    Base class for MVC controllers.  Provides basic controller
    functionality, including credential checking, session management,
    and redirects.
    '''
    
    def __init__(self):

        self.session = None
        self.cookies = None
        self.user = None
        self.error = None

        self.require_auth = True


    def get_user(self, req):

        user = self.check_credentials(req)

        if user is None:
            user = self.check_session(req)

        return user


    def check_credentials(self, req):

        '''Check user credentials.
        For pages where authentication is required, check to see
        if user credentials have been passed in on the request.
        If credentials are present, initialize and return a User object.
        '''

        params = req.params
        user = None

        if 'user_id' in params:
            if 'password' in params:

                user_id = params['user_id']
                password = params['password']

                user = User.get_login_user(user_id, password)

        return user


    def check_session(self, req):

        '''Check session cookie.
        Check request for a session cookie.  If present, check the session
        for a valid user id.  If the user id is valid, initialize and return
        a User object.
        '''

        self.cookies = req.cookies

        if 'session' in self.cookies:
            self.session = Session.get_session(self.cookies['session'])

        if self.session is not None:

            user_id = self.session.uid
            self.user = User.get_user(user_id)

        return self.user
    

    def redirect(self, location):

        req = Request.blank('/', base_url=bivouac.base_url)
        e = exc.HTTPTemporaryRedirect(location=location)
        return req.get_response(e)


    def set_session(self, resp):

        self.session = Session()
        self.session.uid = self.user.id
        self.session.expires = datetime.datetime.now() + datetime.timedelta(days=1) 
        self.session.store()

        resp.set_cookie('session', self.session.id,
                        expires=self.session.expires)

        return self.session



import sys
from xml.dom.minidom import parse
from webob import Request, Response, exc

class View(object):


    def __init__(self, model):
        
        self.model = model
        self.scripts = []
        self.styles = []


    def populate_template(self):
        
        raise NotImplementedError('Method not implemented by inheriting class.')


    @property
    def Model(self):
        
        return self.model;


    @Model.setter
    def Model(self, model):

        self.model = model;


    def __call__(self, environ, start_response):
        
        output = self.populate_template()
        resp = Response(body=output)
        return resp(environ, start_response)


    def __str__(self):
        return self.populate_template()



from string import Template


class TemplateNotFoundException(Exception):

    '''Exception raised when a template is not found within the
    projectli.st application.
    '''

    def __init__(self, value):
        self.value = value


    def __str__(self):
        return repr(self.value)



class TemplateCache(dict):

    _template_dir = '/Users/signal9/Projects/projectli.st/views/templates/'

    def __init__(self):
        dict.__init__(self)


    def load_template(self, template_name):

        '''Load a template from file into local cache.
        '''

        template_file = template_name + '.html'
        template_path = TemplateCache._template_dir + template_file

        try:
            with open(template_path) as template:
                temp = template.read()
                self[template_name] = Template(temp)
        except:
            raise TemplateNotFoundException('Template %s not found.' % template_name)

        return self[template_name]
        


    def get_template(self, template_name):

        '''Retrieve a template from cache, loading if necessary.
        '''

        if template_name in self:
            return self[template_name]
        return self.load_template(template_name)



import hashlib
from pymongo import Connection
from bson import ObjectId


class Session(dict):

    
    '''Session class.
    '''

    def __init__(self):
        dict.__init__(self)
        self._uid = None


    @staticmethod
    def _get_session_collection():

        if not hasattr(Session, '_collection'):

            connection = Connection('localhost', 27017)
            db = connection.projectlist
            Session._collection = db.sessions

        return Session._collection


    @staticmethod
    def get_session(id):

        '''Return a single Session object from the database.
        '''

        sessions = Session._get_session_collection()
        session = sessions.find_one({'_id': ObjectId(id)}, as_class=Session)
        return session


    @property
    def id(self):
        return str(self._id)

    
    @property
    def uid(self):
        return self._uid


    @uid.setter
    def uid(self, uid):
        self._uid = uid

    
    def store(self):

        session_collection = Session._get_session_collection()
        self._id = session_collection.save(self.__dict__, safe=True)
        return self._id


    def delete(self):
        pass


    def __getitem__(self, key):
        if not key in self:
            raise KeyError(key)
        else:
            return getattr(self, key)


    def __setitem__(self, key, value):
        setattr(self, key, value)



from router import Router
from controller import Controller
from user import User
from session import Session
from model import Model
from view import View
from jsonview import JSONView

base_url = ''
template_path = ''



import hashlib
from pymongo import Connection
from bson import ObjectId

class User (dict):

    '''User Object'''

    
    @staticmethod
    def _get_user_collection():

        if not hasattr(User, '_collection'):

            connection = Connection('localhost', 27017)
            db = connection.projectlist
            User._collection = db.users

        return User._collection


    @staticmethod
    def get_active_users():

        users = User._get_user_collection()
        result = users.find(as_class=User)
        return result


    @staticmethod
    def get_user(id):
        
        '''Return a single User object from the database.
        If 'id' is not a valid ObjectId, throws an InvalidId exception
        If 'id' is not present in the database, say from a deleted User record,
        returns a None type.
        '''

        users = User._get_user_collection()
        user = users.find_one({'_id': ObjectId(id)}, as_class=User)
        return user


    @staticmethod
    def get_login_user(user_name, password):

        pwdhash = hashlib.md5(password).hexdigest()

        users = User._get_user_collection()
        user = users.find_one({'login': user_name, '_password': pwdhash}, as_class=User)
        return user

    
    def __init__(self, id=None):
        
        dict.__init__(self)
        self._password = None


    def __getitem__(self, key):
        if not hasattr(self, key):
            raise KeyError(key)
        else:
            return getattr(self, key)


    def __setitem__(self, key, value):
        setattr(self, key, value)


    @property
    def id(self):
        return str(self._id)


    @id.setter
    def id(self, strid):
        self._id = ObjectId(strid)
    

    @property
    def password(self):
        return self._password

    @password.setter
    def password(self, value):
        self._password = hashlib.md5(value).hexdigest()

    @password.deleter
    def password(self):
        del self._password

        
    def store(self):
        
        _id = User._get_user_collection().save(self.__dict__)
        return _id


    def delete(self):
        pass



import sys
from user import User

class Model (object):

    def __init__(self, session=None):

        self.session = session
        self.user = None

    @property
    def User(self):

        if(self.user is None):
            uid = self.session.uid
            self.user = User.get_user(uid)

        return self.user



import distribute_setup
distribute_setup.use_setuptools()

from setuptools import setup, find_packages

setup(
        name='Bivouac',
        version='0.1.0',
        author='Adam A.G. Shamblin',
        author_email='adam.shamblin@gmail.com',
        packages=find_packages(),
        url='http://pypi.python.org/pypi/bivouac',
        license='MIT',
        keywords='web mvc wsgi framework',
        description='a light-weight, wsgi-compliant web framework in Python',
        long_description=open('README.txt').read(),
        install_requires=[
            'distribute',
            'WebOb >= 1.2.1',
            'Paste >= 1.7.5.1',
            'pymongo >= 2.1.1'
        ]
)



import json
import bivouac

class JSONView(bivouac.View):

    '''JSONView.
    A generic view to return JSON serialized objects.
    '''

    def __init__(self, model=None):

        bivouac.View.__init__(self, model)

        self.model = model


    def populate_template(self):

		if type(self.model) == str:
			return self.model
		
		return json.dumps(self.model)



import sys
import re
from pprint import pformat
from webob import Request, exc


class Router(object):

    '''MVC Router class for WSGI applications.
    '''

    route_regex = re.compile('{(\w+)(?::([^}]+))?\}')

    def __init__(self):
        self.routes = []


    def add_route(self, template, defaults=None, **vars):

        '''add_route() - Add a route mapping to the application.

        The routes member is a list of tuples where each tuple contains
        a compiled regex to match against the current URL, the Route object,
        and any supporting arguments.
        '''

        controller = Route(defaults)
        self.routes.append((re.compile(self.template_to_regex(template)),
                            controller,
                            vars))


    def __call__(self, environ, start_response):

        req = Request(environ)
        for regex, controller, vars in self.routes:
            match = regex.match(req.path_info)
            if match:
                req.urlvars = match.groupdict()
                req.urlvars.update(vars)
                return controller(environ, start_response)

        return exc.HTTPNotFound()(environ, start_response)


    def template_to_regex(self, template):

        regex = ''
        last_pos = 0

        for match in Router.route_regex.finditer(template):
            regex += re.escape(template[last_pos:match.start()])
            name = match.group(1)
            expr = match.group(2) or '[^/]+'
            expr = '(?P<%s>%s)' % (name, expr)
            regex += expr
            last_pos = match.end()

        regex += re.escape(template[last_pos:])
        regex = '^%s$' % regex
        return regex


class Route(object):

    '''MVC Route class for WSGI applications.
    '''
    
    def __init__(self, defaults=None):
        self.defaults = defaults


    def __call__(self, environ, start_response):

        req = Request(environ)

        if 'controller' in req.urlvars:
            controller = req.urlvars['controller'].lower()
        else:
            controller = self.defaults['controller'].lower()
        controller = self.load_controller(controller)

        if 'action' in req.urlvars:
            action = req.urlvars['action']
        else:
            action = self.defaults['action']
        action = getattr(controller, action)
        
        return action(environ, start_response)


    def load_controller(self, module_name):

        __import__(module_name)
        module = sys.modules[module_name]
        controller_name = getattr(module, 'controller')
        controller = getattr(module, controller_name)

        return controller()
